/*
 * AxeMain.java
 * v4.0.0 (c)2010-2013 Odin Technology Ltd. All rights reserved.
*/

package helpers;

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.xml.parsers.*;
import javax.xml.xpath.*;
import javax.xml.namespace.NamespaceContext;

import org.w3c.dom.*;

import com.rational.test.ft.object.interfaces.BrowserTestObject;
import com.rational.test.ft.object.interfaces.ProcessTestObject;
import com.rational.test.ft.object.interfaces.TestObject;
import com.rational.test.ft.object.interfaces.TopLevelTestObject;
import com.rational.test.ft.script.IOptionName;
import com.rational.test.ft.script.RationalTestScript;
import com.rational.test.ft.script.State;
import com.rational.test.ft.value.RegularExpression;
import com.jacob.activeX.ActiveXComponent;
import com.jacob.com.Variant;
import com.jacob.com.Dispatch;

public abstract class AxeMain extends RationalTestScript
{
	public static ActiveXComponent axe;
	public static ActiveXComponent harness;
	public static TestObject application;
	public static BrowserTestObject browser;
	public static TestObject htmlDocument;
	
	public static String axeProjectFile = "";
	public static String axeProjectDir = "";
	public static String axeResultFile = "";
	public static String axeRunConfig = "";
	public static String axeTestProject = "";
	public static String axeMasterTestSuite = "";
	public static String axeTestSuite = "";
	public static String axeDefaultRunConfig = "";
	public static String axePostResultsDir = "";
	
	public static String axeRqmExecutionId = "";
		
	public static void axeInit(String testProject, String testSuite, Object[] args)
	{
		logInfo("Datastore=" + (String)getOption(IOptionName.DATASTORE));
		
	     axe = new ActiveXComponent("OdinTechnology.Axe.Runtime.Test"); 
         harness = new ActiveXComponent("OdinTechnology.Axe.Harness.AxeHarnessAPI");
         axeProjectFile = new File(testProject).getName();
         axeTestProject = testProject;
		 axeMasterTestSuite = testSuite;
         axeTestSuite = testSuite;
         axeParseArgs(args);
         if(!axeProjectDir.equals(""))
         {
			axeCreateProject();
			if(!axePostResultsDir.equals(""))
			{
				logInfo("PostResultsDir=" + axePostResultsDir);
				Dispatch.put(axe, "PostResults", true);
				Dispatch.put(axe, "PostResultsDir", axePostResultsDir);
			}
         }
         Dispatch.call(axe, "InitSuite", axeTestProject, axeTestSuite);
         Dispatch.call(harness, "SetSettings", Dispatch.call(axe, "GetRunCategoryOptions", "AxeHarness"));
         axeSetRFTOptions(Dispatch.call(axe, "GetRunCategoryOptions", "RFT").toString());
         String browserName = Dispatch.call(axe, "GetRunCategoryOption", "Browser", "name").toString();
         if(browserName.equals("Firefox") || browserName.equals("Mozilla Firefox"))
         {
        	 setCurrentBrowser("Mozilla Firefox");
         }
	}
	
	public static void axeParseArgs(Object[] args)
	{
		for(int i = 0; i < args.length; i++)
		{
			logInfo("Arg: " + args[i]);	
			int delimitterPos = args[i].toString().indexOf(':');
			if(delimitterPos != -1)
			{
				String argName = args[i].toString().substring(0, delimitterPos).toLowerCase();
				String argValue = args[i].toString().substring(delimitterPos + 1);			
				if(argName.equals("/axeprojectdir")) {
					axeProjectDir = argValue;}
				else if(argName.equals("/axerunconfig")){
					axeRunConfig = argValue;}		
			}
		}
	}
	
	public static void axeCreateProject()
	{	
		logInfo("Creating local Axe project");
		
		String rqmTaskFile = System.getProperty("rqm.task");
        if(rqmTaskFile != null && !rqmTaskFile.isEmpty())
        {
        	logInfo("rqm.task: " + rqmTaskFile);
        	axeGetRqmTaskData(rqmTaskFile);
        }
		
		String localProjectFolder = getOption(IOptionName.DATASTORE) + "/_axe" + axeRqmExecutionId;
		axeTestProject = localProjectFolder + "/" + axeProjectFile;
		axeTestSuite = axeTestSuite + "_" + axeRqmExecutionId;
		String resultsFolder = localProjectFolder + "/results/" + axeTestSuite;
		axeResultFile = resultsFolder + "/results.xml";
		logInfo("Axe results file: " + axeResultFile);
		if(new File(localProjectFolder).exists())
		{
			logInfo("Axe project folder already exists -- " + localProjectFolder);
			axeGetProjectOptions(axeTestProject);			
			return;
		}
			
		if(axeProjectDir.equals(localProjectFolder))
		{
			logError("Local project folder cannot be the same as the source folder.");
			logInfo("Local project folder: " + localProjectFolder);
			return;
		}
		try {
			axeDownloadProject(axeProjectDir, localProjectFolder);
		} catch (IOException e) {
			logError("Fatal error downloading Axe project: " + e.toString());
			e.printStackTrace();
			return;
		}
		
		// Copy results templates
		Document document = null;
		String axeDir = Dispatch.call(axe, "GetEnv", "AXEDIR").toString();
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			document = db.parse(new File(axeDir, "axeconfig.xml"));
		}
		catch(Exception e){
			logError("Cannot load AxeConfig.xml from AXEDIR");
			return;
		}
		try{
			NodeList nodes = document.getElementsByTagName("resulttemplate");
			for(int n=0; n < nodes.getLength(); n++){
				String template = axeDir + "\\templates\\" + nodes.item(n).getTextContent();
				String destination = resultsFolder + "\\" + nodes.item(n).getTextContent();
				copyFile(new File(template), new File(destination));
			}
		}
		catch(Exception e){
			logError("Cannot copy Axe result templates");
		}
		
		// Create run file
		axeGetProjectOptions(axeTestProject);
		// if no RunConfig was provided and we have a default value then use the default
		if(axeRunConfig.equals("") && !axeDefaultRunConfig.equals("")){
			axeRunConfig = axeDefaultRunConfig;
		}
		String runConfigFile = localProjectFolder + "/config/" + axeRunConfig + ".RunConfig.xml";
		if(!new File(runConfigFile).exists())
		{
			runConfigFile = localProjectFolder + "/config/" + axeRunConfig + ".RunConfig.xlsx";
			if(!new File(runConfigFile).exists())
				logError("No Run Config file found -- " + axeRunConfig);
			}
		String runFilename = resultsFolder + "/run.xml";
		String runName = axeRqmExecutionId;
		axeCreateRunFile(runFilename, runConfigFile, runName);			
	}
	
	public static void axeCreateRunFile(String filename, String runConfig, String runName)
	{
		try{
			BufferedWriter runFile = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename), "UTF8"));
			runFile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
			runFile.write("<RunSettings>");
			runFile.write("  <Option Name=\"CheckDependency\" Value=\"True\" />");
			if(new File(runConfig).exists()){
				logInfo("RunConfig=" + runConfig);
				runFile.write("  <Option Name=\"RunConfiguration\" Value=\"" + runConfig + "\" />");
			}
			runFile.write("  <Option Name=\"RunName\" Value=\"" + runName + "\" />");
			runFile.write("</RunSettings>");
			runFile.close();
		}
		catch(Exception e)
		{	logInfo("Fatal error creating run.xml: " + e.getMessage());}
	}
	
	public static void axeGetProjectOptions(String projectFile)
	{
		String rqmVersion = "RQM4";
		Document document = null;
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			document = db.parse(new File(projectFile));
		}
		catch(Exception e){
			logInfo("Cannot load Axe project file -- " + projectFile);
		}
		try{
			XPathFactory xPathfactory = XPathFactory.newInstance();
			XPath xpath = xPathfactory.newXPath();
			XPathExpression expr = xpath.compile("/TestProject/AxeManager/ALM[@Name='" + rqmVersion + "']/Options/Option");
			NodeList nodes = (NodeList)expr.evaluate(document, javax.xml.xpath.XPathConstants.NODESET);
			if(nodes.getLength() == 0) {
				rqmVersion = "RQM3";
				expr = xpath.compile("/TestProject/AxeManager/ALM[@Name='" + rqmVersion + "']/Options/Option");
				nodes = (NodeList)expr.evaluate(document, javax.xml.xpath.XPathConstants.NODESET);
			}
			for(int n=0; n < nodes.getLength(); n++){
				Element e = (Element)nodes.item(n);
				if(e.getAttribute("name").equals("DefaultRunConfig"))
				{
					axeDefaultRunConfig = e.getAttribute("value");
				}
				else if(e.getAttribute("name").equals("PostResultsDir"))
				{
					axePostResultsDir = e.getAttribute("value");
				}
			}
		}
		catch(Exception e){
			logInfo("Cannot load AxeManager options for Axe project file");
		}	
	}
	
	public static void axeGetRqmTaskData(String taskFilename)
	{
		File taskFile = new File(taskFilename);
		File localTaskFile = new File(System.getProperty("user.dir") + "/" + taskFile.getName());
		if(!taskFile.getAbsolutePath().equals(localTaskFile.getAbsolutePath()))
		{
			try{
			copyFile(taskFile, localTaskFile);
			}
			catch(IOException e){
				logError("Cannot create local copy of task file");
			}
			logInfo("Local copy of RQM task file: " + localTaskFile.getAbsolutePath());
		}
					
		Document document = null;
		try{
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			document = db.parse(taskFile);
		}
		catch(Exception e){
			logError("Cannot load RQM task file -- " + taskFilename);
		}
		try{
			XPathFactory xPathfactory = XPathFactory.newInstance();
			XPath xpath = xPathfactory.newXPath();
			XPathExpression expr = xpath.compile("//variables/variable/name");
			NodeList nodes = (NodeList)expr.evaluate(document, javax.xml.xpath.XPathConstants.NODESET);
			for(int n=0; n < nodes.getLength(); n++){
				Element e = (Element)nodes.item(n);
				if(e.getTextContent().toLowerCase().equals("axerunconfig"))
				{
					expr = xpath.compile("../value");
					nodes = (NodeList)expr.evaluate(e, javax.xml.xpath.XPathConstants.NODESET);			
					String value = ((Element)nodes.item(0)).getTextContent();
					if(!value.isEmpty()){
						axeRunConfig = value;				}					
				}		
			}	

			String taskFileContents = readFile(taskFilename);
			Integer index = taskFileContents.indexOf("suiteLogURL");
			if(index > 0)
			{
				index = taskFileContents.indexOf("testsuitelog:");
				String testsuitelog = taskFileContents.substring(index + 13);
				testsuitelog = testsuitelog.substring(0, testsuitelog.indexOf("\""));
				logInfo("testsuitelog: " + testsuitelog);
				axeRqmExecutionId = "S" + testsuitelog;
				logInfo("RFT is running from RQM in suite mode.");
			}
			else
			{
				index = taskFileContents.indexOf("tasks:");
				String tasks = taskFileContents.substring(index + 6);
				tasks = tasks.substring(0, tasks.indexOf("<"));
				logInfo("tasks: " + tasks);
				axeRqmExecutionId = "T" + tasks;		
				logInfo("RFT is running from RQM in single test mode");
			}
			logInfo("axeRqmExecutionId=" + axeRqmExecutionId);					
		}
		catch(Exception e){
			logInfo("Error reading RQM task file");
			logInfo(e.toString());
		}	
	}
	
	
	public static void axeDownloadProject(String sourceFolder, String targetFolder) throws IOException
	{
		logInfo("Download project from " + sourceFolder + " to " + targetFolder);
		File projectFolder = new File(targetFolder);
		projectFolder.mkdir();
		if(!projectFolder.exists())
		{
			throw new IOException("Could not create directory -- " + projectFolder.getAbsolutePath());
		}
		axeTestProject = targetFolder + "/" + axeProjectFile;
		copyFile(new File(sourceFolder, axeProjectFile), new File(axeTestProject));
	
		copyDirectory(new File(sourceFolder, "config"), new File(targetFolder, "config"));
		copyDirectory(new File(sourceFolder, "data"), new File(targetFolder, "data"));
		File libFolder = new File(sourceFolder, "lib");
		if(libFolder.exists()){
			copyDirectory(libFolder, new File(targetFolder, "lib"));}
		File resultsFolder = new File(targetFolder, "results");
		resultsFolder.mkdir();
		copyDirectory(new File(sourceFolder, new File("results", axeMasterTestSuite).getPath()), new File(resultsFolder, axeTestSuite));
		
		// special case for OdinPortal sample
		File htmlFolder = new File(sourceFolder, "html");
		if(htmlFolder.exists()){
			copyDirectory(htmlFolder, new File(targetFolder, "html"));}
	}
	
	public static void axeSetRFTOptions(String optionsList)
	{
		if(optionsList.equals(""))
		{
			return;
		}
		String options[] = optionsList.split(";");
		String[] kvp;
		String optionName;
		String optionValue;
		for(String option : options)
		{
			kvp = option.split("=");
			optionName = kvp[0].toUpperCase();
			optionValue = kvp[1];
			logInfo(optionName + "=" + optionValue);
			if(optionName.equals("AMBIGUOUS_RECOGNITION_THRESHOLD"))
				setOption(IOptionName.AMBIGUOUS_RECOGNITION_THRESHOLD, Integer.parseInt(optionValue));
			else if(optionName.equals("DELAY_AFTER_WINDOW_ACTIVATE"))
				setOption(IOptionName.DELAY_AFTER_WINDOW_ACTIVATE, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_GUI_ACTION"))
				setOption(IOptionName.DELAY_BEFORE_GUI_ACTION, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_KEY_DOWN"))
				setOption(IOptionName.DELAY_BEFORE_KEY_DOWN, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_KEY_UP"))
				setOption(IOptionName.DELAY_BEFORE_KEY_UP, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_MOUSE_DOWN"))
				setOption(IOptionName.DELAY_BEFORE_MOUSE_DOWN, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_MOUSE_MOVE"))
				setOption(IOptionName.DELAY_BEFORE_MOUSE_MOVE, Double.parseDouble(optionValue));
			else if(optionName.equals("DELAY_BEFORE_MOUSE_UP"))
				setOption(IOptionName.DELAY_BEFORE_MOUSE_UP, Double.parseDouble(optionValue));
			else if(optionName.equals("FIND_OBJECT_DELAY_BETWEEN_RETRIES"))
				setOption(IOptionName.FIND_OBJECT_DELAY_BETWEEN_RETRIES, Double.parseDouble(optionValue));
			else if(optionName.equals("INTERACTIVE"))
				setOption(IOptionName.INTERACTIVE, Boolean.parseBoolean(optionValue));
			else if(optionName.equals("LOG_APPLICATION_GUI_ACTION"))
				setOption(IOptionName.LOG_APPLICATION_GUI_ACTION, Boolean.parseBoolean(optionValue));
			else if(optionName.equals("LOG_EXCEPTION_SNAPSHOT"))
				setOption(IOptionName.LOG_EXCEPTION_SNAPSHOT, Boolean.parseBoolean(optionValue));
			else if(optionName.equals("LOG_FORMAT"))
				setOption(IOptionName.LOG_FORMAT, optionValue);
			else if(optionName.equals("MAXIMUM_FIND_OBJECT_TIME"))
				setOption(IOptionName.MAXIMUM_FIND_OBJECT_TIME, Double.parseDouble(optionValue));
			else if(optionName.equals("MAXIMUM_WAIT_FOR_EXISTENCE"))
				setOption(IOptionName.MAXIMUM_WAIT_FOR_EXISTENCE, Double.parseDouble(optionValue));
			else if(optionName.equals("MOUSE_HOVER_TIME"))
				setOption(IOptionName.MOUSE_HOVER_TIME, Double.parseDouble(optionValue));
			else if(optionName.equals("RECOGNITION_THRESHOLD"))
				setOption(IOptionName.RECOGNITION_THRESHOLD, Integer.parseInt(optionValue));
			else if(optionName.equals("RECOGNITION_THRESHOLD_WARNING"))
				setOption(IOptionName.RECOGNITION_THRESHOLD_WARNING, Integer.parseInt(optionValue));
			else if(optionName.equals("RECOGNITION_THRESHOLD_LASTCHANCE"))
				setOption(IOptionName.RECOGNITION_THRESHOLD_LASTCHANCE, Integer.parseInt(optionValue));
			else if(optionName.equals("TIME_MULTIPLIER"))
				setOption(IOptionName.TIME_MULTIPLIER, Double.parseDouble(optionValue));
			else if(optionName.equals("USE_DEFAULT_LOG_INFO"))
				setOption(IOptionName.USE_DEFAULT_LOG_INFO, Boolean.parseBoolean(optionValue));
			else if(optionName.equals("WAIT_FOR_EXISTENCE_DELAY_BETWEEN_RETRIES"))
				setOption(IOptionName.WAIT_FOR_EXISTENCE_DELAY_BETWEEN_RETRIES, Double.parseDouble(optionValue));
			else
				logError("Invalid option name -- " + optionName);						
		}
	}

	public static void axeLogMessage(String message)
	{
		logInfo(message);	
		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		java.util.Date date = new java.util.Date();
		Dispatch.call(axe, "StepInfoEx", "[" + dateFormat.format(date) + "] " + message);	
	}
	
	public static void axeStepEnd()
	{
		int resultCode = Dispatch.get(axe, "ResultCode").getInt();
		Dispatch.call(axe, "StepEnd", resultCode, "");
		if(resultCode != 0)		
			logTestResult("Step", false);
	}
	
	public static void axeSubtestEnd()
	{
		Dispatch.call(axe, "SubtestEnd");
		for(TestObject to : getRegisteredTestObjects())
		{      
			if(!to.equals(application) && !to.equals(browser))
          		to.unregister();  
		}	         
	}
	
	public static void 	axeTestBegin(String testId, String description, String tableFile, String tableName, String resultsFile, String executionId)
	{
		if(axeResultFile.equals("")) {
			axeResultFile = resultsFile;
		}
		Dispatch.call(axe, "TestBegin", testId, description, tableFile, tableName, axeResultFile, executionId);	
	}
	
	public static void axeTestAbort(String message)
	{
		Dispatch.call(axe, "StepInfoEx", message);
		Dispatch.call(axe, "TestAbortEx");
		logTestResult("TestAbort", false, message);
	}
	
	public static TestObject axeFind(String className, String properties) throws Exception
	{
		return axeFind(null, className, properties);
	}
	 
	public static TestObject axeFind(TestObject root, String className, String properties) throws Exception
	{
		String[] kvp = properties.split("=");
		String propertyName = kvp[0];
		String propertyValue = kvp[1];			
		TestObject[] testObjects;
		RegularExpression regex = null;
		
		if(propertyName.endsWith("_regex"))
		{
			regex = new RegularExpression(propertyValue, true);
			propertyName = propertyName.substring(0, propertyName.indexOf("_regex"));
		}
		
		if(root == null)
		{
			if(regex == null)
				testObjects = find(atDescendant(".class", className, propertyName, propertyValue));
			else
				testObjects = find(atDescendant(".class", className, propertyName, regex));
		}
		else
		{
			if(regex == null)
			{
				if(className == "Html.HtmlDocument")
					testObjects = root.find(atChild(".class", className, propertyName, propertyValue));
				else
					testObjects = root.find(atDescendant(".class", className, propertyName, propertyValue));
			}
			else
			{
				if(className == "Html.HtmlDocument")
					testObjects = root.find(atChild(".class", className, propertyName, regex));
				else
					testObjects = root.find(atDescendant(".class", className, propertyName, regex));
			}
		}
		if(testObjects.length == 0)
		{
			throw new Exception("Object not found. Class: " + className + " Properties: " + properties);	
		}
		if(testObjects.length > 1)
		{
			throw new Exception("Object not unique. Class: " + className + " Properties: " + properties);	
		}
		return testObjects[0];
	}
	 
	public static BrowserTestObject axeBrowserBasestate(String url) throws Exception
	{
		TestObject browsers[] = find(atChild(".class" , "Html.HtmlBrowser"));
		if(browsers.length != 1)
		{
			return axeBrowserRestart(url);
		}
		BrowserTestObject browser = (BrowserTestObject)browsers[0];
		String currentUrl = browser.getProperty(".documentName").toString();
		axeLogMessage("Found browser " + currentUrl);
		if(!currentUrl.equals(url))
		{
			axeLogMessage("Navigating to " + url);
			browser.loadUrl(url);
		}
		return browser;
	}
	
	public static BrowserTestObject axeBrowserRestart(String url) throws Exception
	{
		axeBrowserCloseAll();
		ProcessTestObject browserProcess = startBrowser(url);
		browserProcess.waitForExistence();
		java.util.Calendar start = java.util.Calendar.getInstance(); 
		while((java.util.Calendar.getInstance().getTimeInMillis() - start.getTimeInMillis())/1000 < ((Number)getOption(IOptionName.MAXIMUM_WAIT_FOR_EXISTENCE)).longValue())
		{
			TestObject browsers[] = find(atChild(".class" , "Html.HtmlBrowser"));
			if(browsers.length > 0)
			{
				BrowserTestObject browser = (BrowserTestObject)browsers[0];					
				return browser;
			}
			sleep(1);
		}					
		throw new Exception("Browser not found.");	
	}
	
	public static void axeBrowserCloseAll() throws Exception
	{
		int closeWait = 0;
		for(TestObject browser : find(atChild(".class" , "Html.HtmlBrowser")))
		{
			axeLogMessage("Closing " + ((BrowserTestObject)browser).getProperty(".documentName").toString());
			((BrowserTestObject)browser).close();
			sleep(1);
			while(((BrowserTestObject)browser).exists())
			{
				if(closeWait++ > 10)
				{
					throw new Exception( "Timed-out waiting for browser to close." );
				}
				sleep(1);
			}
		}
	}
	
	public static void axeBrowserWait() throws Exception
	{
		long startTimeInSeconds = System.currentTimeMillis()/1000;
		while(Integer.parseInt(browser.getProperty(".readyState").toString()) < 4)
		{   	 
			if((System.currentTimeMillis()/1000 - startTimeInSeconds) > ((Double)RationalTestScript.getOption(IOptionName.MAXIMUM_WAIT_FOR_EXISTENCE)).doubleValue())
			{
				throw new Exception( "Timed-out waiting for browser to be ready." );
			}
			sleep(1);
		}
	}
	
	
	public static State axeConvertDataToState(String data)
	{
		if(data.equals("") || data.equals("1"))
		{
			return State.selected();
		}
		else if(data.equals("0"))
		{
			return State.notSelected();
		}
		return State.indeterminate();
	}


	// File IO utilities
	public static void copyDirectory(File srcDir, File dstDir) throws IOException {
		//logInfo("copyDirectory " + srcDir + " " + dstDir);
	    if (srcDir.isDirectory()) {
	        if (!dstDir.exists()) {
	            dstDir.mkdir();
	        }
	        String[] children = srcDir.list();
	        for (int i=0; i<children.length; i++) {
	            copyDirectory(new File(srcDir, children[i]), new File(dstDir, children[i]));
	        }
	    } else {
	        copyFile(srcDir, dstDir);
	    }
	}
	
	public static void copyFile(File src, File dst) throws IOException {
		//logInfo("copyFile " + src + " " + dst);
	    InputStream in = new FileInputStream(src);
	    OutputStream out = new FileOutputStream(dst);
	    byte[] buf = new byte[1024];
	    int len;
	    while ((len = in.read(buf)) > 0) {
	        out.write(buf, 0, len);
	    }
	    in.close();
	    out.close();
	}
	
	public static String readFile(String path) throws IOException {
		  FileInputStream stream = new FileInputStream(new File(path));
		  try {
		    java.nio.channels.FileChannel fc = stream.getChannel();
		    java.nio.MappedByteBuffer bb = fc.map(java.nio.channels.FileChannel.MapMode.READ_ONLY, 0, fc.size());
		    return java.nio.charset.Charset.defaultCharset().decode(bb).toString();
		  }
		  finally {
		    stream.close();
		  }
		}

}

